/*
 * Project name:
       CO click
 * Copyright:
      (c) mikroElektronika, 2014.
 * Revision History:
      20140703:
      - Initial release (PT);
 * Description:
      This is a example which demonstrates the use of CO click board with MQ-7 sensor.
      The MQ-7 is a heater-driven CO sensor that outputs an analog signal.
      This CO sensor is suitable for gas detecting equipment for Carbon Monoxide (CO).
      It has a fast response time and high sensitivity. 
      LCD shows PPM value of Carbon Monoxide (CO) concentration.

      For accurate measuring, gas detector should be considering the temperature and humidity influence.
      For more informations about calibration and accurate measuring please see MQ-7 sensor datasheet and sensor manufacturer recommendations.

      For accurate reading sensor requires pre heating time no less than 48 hours.

      MQ-7 sensor informations:
      http://english.hwsensor.com/English/Gas-Sensor-and-Module/gas2011124120328.shtml
 * Test configuration:
     MCU:             PIC18F45K22
                      http://ww1.microchip.com/downloads/en/DeviceDoc/41412F.pdf
     Dev. Board:      EasyPIC v7 Connectivity
                      http://www.mikroe.com/easypic/
     Oscillator:      HS-PLL, 8.000 MHz Crystal, 32.000 MHz MCU Clock
     ext. modules:    CO click
                      http://www.mikroe.com/click/co/
     SW:              mikroC PRO for PIC
                      http://www.mikroe.com/mikroc/pic/
 * NOTES:
      - Place CO Click board in the mikroBUS socket 1.
      - Turn on Lcd backlight switch SW4.6. (board specific)
*/

// Lcd module connections
sbit LCD_RS at LATB4_bit;
sbit LCD_EN at LATB5_bit;
sbit LCD_D4 at LATB0_bit;
sbit LCD_D5 at LATB1_bit;
sbit LCD_D6 at LATB2_bit;
sbit LCD_D7 at LATB3_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;
// End Lcd module connections

const double Rl      = 5000.0;               // Rl (Ohm) - Load resistance
const double Vadc_5  = 0.0048828125;         // ADC step 5V/1024 4,88mV (10bit ADC)
const double Vadc_33 = 0.0032226562;         // ADC step 3,3V/1024 3,22mV (10bit ADC)
double Vrl;                                  // Output voltage
double Rs;                                   // Rs (Ohm) - Sensor resistance
double ppm;                                  // ppm
double ratio;                                // Rs/Rl ratio

unsigned int adc_rd;
char txt[16];


//Calculation of PPM
void calculatePPM() {
  double lgPPM;
  Vrl = (double)adc_rd * Vadc_5;             // For 5V Vcc use Vadc_5  and  for 3V Vcc use Vadc_33
  Rs = Rl * (5 - Vrl)/Vrl;                   // Calculate sensor resistance
  ratio = Rs/Rl;                             // Calculate ratio
  lgPPM = (log10(ratio) * -3.7)+ 0.9948;     // Calculate ppm
  ppm = pow(10,lgPPM);                       // Calculate ppm
}

//Read sensor
void readSensor() {
  adc_rd = ADC_Read(2);                      // Read RA2
  delay_ms(10);                              // Pause 10ms
}

void main() {

  LCD_Init();                                // Initialize Lcd
  Lcd_Cmd(_LCD_CLEAR);                       // Clear display
  Lcd_Cmd(_LCD_CURSOR_OFF);                  // Cursor off
  Lcd_Out(1,1,"CO click");                   // Write text in first row

  ANSELA = 0x04;                             // Set ADC on RA2
  TRISA = 0x04;                              // Set RA2 as input
  LATA = 0;                                  // Set PORTA as 0
  ADC_Init();                                // Initialize ADC
  delay_ms(100);                             // Pause of 100ms for ADC module stabilization

  //Initial read ADC and display PPM value on LCD
  readSensor();                              // Read sensor
  calculatePPM();                            // Calculating PPM value
  
  FloatToStr(ppm,&txt);                      // Conversion float ppm to string txt
  LCD_Out(2,14,"ppm");                       // Print text on LCD
  LCD_Out(2,1,txt);                          // Print txt value on LCD
  
  while(1) {
  
    delay_ms(500);                           // Pause 500ms
    readSensor();                            // Read sensor
    calculatePPM();                          // Calculating PPM value
    FloatToStr(ppm,&txt);                    // Conversion float ppm to string txt
    LCD_Out(2,1,txt);                        // Print txt value on LCD
    
  }
}